# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import os.path
import ConfigParser
import string
import sys

CONF_TYPE_INTEGER        = 1 # An integer
CONF_TYPE_BOOLEAN        = 2 # A boolean (True, False, Yes, No, 1, 0)
CONF_TYPE_FLOAT          = 3 # A float
CONF_TYPE_STRING         = 4 # A string
CONF_TYPE_DIR            = 5 # One directory
CONF_TYPE_PATH           = 6 # Multiple directories
CONF_TYPE_FILE           = 7 # One file
CONF_TYPE_MULTIPLE_FILES = 8 # Multiple files
CONF_TYPE_CHOICE         = 9 # Choose one value from a list.

# {"section": [(varname, description, type, defaultvalue, extra_info), ...], ...}
configuration_info = {
    "General": [("hrf directories", "Kataloger med hrf-filer i",
                 CONF_TYPE_PATH, [], None),
                ("lineup direction", "Riktning fr laguppstllningar",
                 CONF_TYPE_CHOICE, "Up", ["Up", "Down", "Left", "Right"])],
    "Fetch": [("save directory", "Katalog att spara hrf-filer i",
               CONF_TYPE_DIR, os.path.realpath("."), None),
              ("download at startup", "Ladda ner vid start av Mattricks",
               CONF_TYPE_BOOLEAN, False, None),
              ("always download", "Ladda alltid ner ny hrf-fil",
               CONF_TYPE_BOOLEAN, True, None),
              ("override old hrf", "Skriv ver dags-gammal hrf-fil om sdan finns",
               CONF_TYPE_BOOLEAN, False, None),
              ("junior spy", "Anvnd Junior Spy",
               CONF_TYPE_BOOLEAN, False, None),
              ("ip number", "IP-nummer fr servern",
               CONF_TYPE_CHOICE, "195.149.159.156", ["195.149.159.155", "195.149.159.156", "195.149.159.157"])]
    }

class Config:
    def __init__(self, config_info):
        self.config_info = config_info

        self.config_file = self.__get_filename()
        if os.path.exists(self.config_file):
            self.configuration = self.__read_config_file()
        else:
            self.configuration = self.__defaults()

    def save(self):
        ci = self.config_info.items()
        ci.sort()

        file = open(self.config_file, 'w')

        first = True
        for section, vars in ci:
            if first:
                first = False
            else:
                file.write("\n")
            file.write("[%s]\n" % section)
            for varname, descr, type, default, extra_info in vars:
                file.write("%s = %s\n" % \
                           (varname, self.__convert_value_to_string \
                            (type, self.configuration[section][varname])))

    def get(self, section, option):
        return self.configuration[section][option]

    def set(self, section, option, value):
        self.configuration[section][option] = value

    def __get_filename(self):
        home = None

        if sys.platform.startswith("win"):
            fname = "mattricks.rc"
            add_dir = True
            try:
                from win32com.shell import shell, shellcon
                home = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, None, 0)
            except ImportError, e:
                pass
        else:
            fname = ".mattricksrc"
            add_dir = False

        if home == None:
            home = os.getenv("HOME")
        if home == None:
            home = ""
        else:
            if add_dir:
                home = os.path.join(home, "Mattricks")
                if not os.path.exists(home):
                    os.makedirs(home)

        return os.path.join(home, fname)

    def __read_config_file(self):
        c = ConfigParser.ConfigParser()
        c.read(self.config_file)

        result = {}

        for section, vars in self.config_info.items():
            varlist = {}
            if not c.has_section(section):
                for varname, descr, type, default, extra_info in vars:
                    varlist[varname] = default
            else:
                for varname, descr, type, default, extra_info in vars:
                    if c.has_option(section, varname):
                        try:
                            varlist[varname] = self.__convert_string_to_value \
                                               (type, c.get(section, varname), extra_info)
                        except ValueError, error:
                            sys.stderr.write("Error in config file in section \"%s\"," \
                                             " option \"%s\":\n   %s\n" % \
                                             (section, varname, str(error)))
                            varlist[varname] = default
                    else:
                        varlist[varname] = default
            result[section] = varlist
        return result

    def __defaults(self):
        result = {}

        for section, vars in self.config_info.items():
            varlist = {}
            for varname, descr, type, default, extra_info in vars:
                varlist[varname] = default
            result[section] = varlist
        return result

    def __convert_value_to_string(self, type, value):
        if type == CONF_TYPE_BOOLEAN:
            if value:
                return "Yes"
            else:
                return "No"
        elif type == CONF_TYPE_FLOAT:
            return "%.4f" % value
        elif type in [CONF_TYPE_PATH, CONF_TYPE_MULTIPLE_FILES]:
            if len(value) == 0:
                return ""
            else:
                if sys.platform.startswith("win"):
                    return string.join(value, ";")
                else:
                    return string.join(value, ":")
        else:
            return str(value)

    def __convert_string_to_value(self, type, valstr, extra_info = None):
        if type == CONF_TYPE_INTEGER:
            return int(valstr)
        elif type == CONF_TYPE_BOOLEAN:
            if valstr.lower() in ["1", "true", "yes"]:
                return True
            elif valstr.lower() in ["0", "false", "no"]:
                return False
            else:
                raise ValueError("invalid literal for CONF_TYPE_BOOLEAN: %s" % valstr)
        elif type == CONF_TYPE_FLOAT:
            return float(valstr)
        elif type in [CONF_TYPE_PATH, CONF_TYPE_MULTIPLE_FILES]:
            if len(valstr) == 0:
                return []
            else:
                if sys.platform.startswith("win"):
                    return string.split(valstr, ";")
                else:
                    return string.split(valstr, ":")
        elif type == CONF_TYPE_CHOICE:
            if not valstr.capitalize() in extra_info:
                raise ValueError("invalid literal for CONF_TYPE_CHOICE: %s" % valstr)
            return valstr
        else:
            return valstr

config = Config(configuration_info)
